@@ -8,11 +8,9 @@ var inherits = require('util').inherits,
   debugOptions = require('../connection/utils').debugOptions,
   retrieveBSON = require('../connection/utils').retrieveBSON,
   Pool = require('../connection/pool'),
-  Query = require('../connection/commands').Query,
   MongoError = require('../error').MongoError,
   MongoNetworkError = require('../error').MongoNetworkError,
-  TwoSixWireProtocolSupport = require('../wireprotocol/2_6_support'),
-  ThreeTwoWireProtocolSupport = require('../wireprotocol/3_2_support'),
+  wireProtocol = require('../wireprotocol'),
   BasicCursor = require('../cursor'),
   sdam = require('./shared'),
   createClientInfo = require('./shared').createClientInfo,
@@ -23,57 +21,6 @@ var inherits = require('util').inherits,
 
 const collationNotSupported = require('../utils').collationNotSupported;
 
-function getSaslSupportedMechs(options) {
-  if (!options) {
-    return {};
-  }
-
-  const authArray = options.auth || [];
-  const authMechanism = authArray[0] || options.authMechanism;
-  const authSource = authArray[1] || options.authSource || options.dbName || 'admin';
-  const user = authArray[2] || options.user;
-
-  if (typeof authMechanism === 'string' && authMechanism.toUpperCase() !== 'DEFAULT') {
-    return {};
-  }
-
-  if (!user) {
-    return {};
-  }
-
-  return { saslSupportedMechs: `${authSource}.${user}` };
-}
-
-function getDefaultAuthMechanism(ismaster) {
-  if (ismaster) {
-    // If ismaster contains saslSupportedMechs, use scram-sha-256
-    // if it is available, else scram-sha-1
-    if (Array.isArray(ismaster.saslSupportedMechs)) {
-      return ismaster.saslSupportedMechs.indexOf('SCRAM-SHA-256') >= 0
-        ? 'scram-sha-256'
-        : 'scram-sha-1';
-    }
-
-    // Fallback to legacy selection method. If wire version >= 3, use scram-sha-1
-    if (ismaster.maxWireVersion >= 3) {
-      return 'scram-sha-1';
-    }
-  }
-
-  // Default for wireprotocol < 3
-  return 'mongocr';
-}
-
-function extractIsMasterError(err, result) {
-  if (err) {
-    return err;
-  }
-
-  if (result && result.result && result.result.ok === 0) {
-    return new MongoError(result.result);
-  }
-}
-
 // Used for filtering out fields for loggin
 var debugFields = [
   'reconnect',
@@ -90,7 +37,6 @@ var debugFields = [
   'connectionTimeout',
   'checkServerIdentity',
   'socketTimeout',
-  'singleBufferSerializtion',
   'ssl',
   'ca',
   'crl',
@@ -225,9 +171,6 @@ var Server = function(options) {
   this.monitoringProcessId = null;
   // Initial connection
   this.initialConnect = true;
-  // Wire protocol handler, default to oldest known protocol handler
-  // this gets changed when the first ismaster is called.
-  this.wireProtocolHandler = new TwoSixWireProtocolSupport();
   // Default type
   this._type = 'server';
   // Set the client info
@@ -303,20 +246,6 @@ Object.defineProperty(Server.prototype, 'name', {
   }
 });
 
-function isSupportedServer(response) {
-  return response && typeof response.maxWireVersion === 'number' && response.maxWireVersion >= 2;
-}
-
-function configureWireProtocolHandler(self, ismaster) {
-  // 3.2 wire protocol handler
-  if (ismaster.maxWireVersion >= 4) {
-    return new ThreeTwoWireProtocolSupport();
-  }
-
-  // default to 2.6 wire protocol handler
-  return new TwoSixWireProtocolSupport();
-}
-
 function disconnectHandler(self, type, ns, cmd, options, callback) {
   // Topology is not connected, save the call in the provided store to be
   // Executed at some point when the handler deems it's reconnected
@@ -344,16 +273,13 @@ function monitoringProcess(self) {
     // Emit monitoring Process event
     self.emit('monitoring', self);
     // Perform ismaster call
-    // Query options
-    var queryOptions = { numberToSkip: 0, numberToReturn: -1, checkKeys: false, slaveOk: true };
-    // Create a query instance
-    var query = new Query(self.s.bson, 'admin.$cmd', { ismaster: true }, queryOptions);
     // Get start time
     var start = new Date().getTime();
 
     // Execute the ismaster query
-    self.s.pool.write(
-      query,
+    self.command(
+      'admin.$cmd',
+      { ismaster: true },
       {
         socketTimeout:
           typeof self.s.options.connectionTimeout !== 'number'
@@ -361,7 +287,7 @@ function monitoringProcess(self) {
             : self.s.options.connectionTimeout,
         monitoring: true
       },
-      function(err, result) {
+      (err, result) => {
         // Set initial lastIsMasterMS
         self.lastIsMasterMS = new Date().getTime() - start;
         if (self.s.pool.isDestroyed()) return;
@@ -377,7 +303,7 @@ function monitoringProcess(self) {
 }
 
 var eventHandler = function(self, event) {
-  return function(err) {
+  return function(err, conn) {
     // Log information of received information if in info mode
     if (self.s.logger.isInfo()) {
       var object = err instanceof MongoError ? JSON.stringify(err) : {};
@@ -388,126 +314,67 @@ var eventHandler = function(self, event) {
 
     // Handle connect event
     if (event === 'connect') {
-      // Issue an ismaster command at connect
-      // Query options
-      var queryOptions = { numberToSkip: 0, numberToReturn: -1, checkKeys: false, slaveOk: true };
-      // Create a query instance
-      var compressors =
-        self.s.compression && self.s.compression.compressors ? self.s.compression.compressors : [];
-      var query = new Query(
-        self.s.bson,
-        'admin.$cmd',
-        Object.assign(
-          { ismaster: true, client: self.clientInfo, compression: compressors },
-          getSaslSupportedMechs(self.s.options)
-        ),
-        queryOptions
-      );
-      // Get start time
-      var start = new Date().getTime();
-      // Execute the ismaster query
-      self.s.pool.write(
-        query,
-        {
-          socketTimeout: self.s.options.connectionTimeout || 2000
-        },
-        function(err, result) {
-          // Set initial lastIsMasterMS
-          self.lastIsMasterMS = new Date().getTime() - start;
+      self.initialConnect = false;
+      self.ismaster = conn.ismaster;
+      self.lastIsMasterMS = conn.lastIsMasterMS;
+      if (conn.agreedCompressor) {
+        self.s.pool.options.agreedCompressor = conn.agreedCompressor;
+      }
 
-          const serverError = extractIsMasterError(err, result);
+      if (conn.zlibCompressionLevel) {
+        self.s.pool.options.zlibCompressionLevel = conn.zlibCompressionLevel;
+      }
 
-          if (serverError) {
-            self.destroy();
-            return self.emit('error', serverError);
-          }
+      if (conn.ismaster.$clusterTime) {
+        const $clusterTime = conn.ismaster.$clusterTime;
+        self.clusterTime = $clusterTime;
+      }
 
-          if (!isSupportedServer(result.result)) {
-            self.destroy();
-            const latestSupportedVersion = '2.6';
-            const message =
-              'Server at ' +
-              self.s.options.host +
-              ':' +
-              self.s.options.port +
-              ' reports wire version ' +
-              (result.result.maxWireVersion || 0) +
-              ', but this version of Node.js Driver requires at least 2 (MongoDB' +
-              latestSupportedVersion +
-              ').';
-            return self.emit('error', new MongoError(message), self);
-          }
+      // It's a proxy change the type so
+      // the wireprotocol will send $readPreference
+      if (self.ismaster.msg === 'isdbgrid') {
+        self._type = 'mongos';
+      }
 
-          // Determine whether the server is instructing us to use a compressor
-          if (result.result && result.result.compression) {
-            for (var i = 0; i < self.s.compression.compressors.length; i++) {
-              if (result.result.compression.indexOf(self.s.compression.compressors[i]) > -1) {
-                self.s.pool.options.agreedCompressor = self.s.compression.compressors[i];
-                break;
-              }
+      // Have we defined self monitoring
+      if (self.s.monitoring) {
+        self.monitoringProcessId = setTimeout(monitoringProcess(self), self.s.monitoringInterval);
+      }
+
+      // Emit server description changed if something listening
+      sdam.emitServerDescriptionChanged(self, {
+        address: self.name,
+        arbiters: [],
+        hosts: [],
+        passives: [],
+        type: sdam.getTopologyType(self)
+      });
+
+      if (!self.s.inTopology) {
+        // Emit topology description changed if something listening
+        sdam.emitTopologyDescriptionChanged(self, {
+          topologyType: 'Single',
+          servers: [
+            {
+              address: self.name,
+              arbiters: [],
+              hosts: [],
+              passives: [],
+              type: sdam.getTopologyType(self)
             }
+          ]
+        });
+      }
 
-            if (self.s.compression.zlibCompressionLevel) {
-              self.s.pool.options.zlibCompressionLevel = self.s.compression.zlibCompressionLevel;
-            }
-          }
+      // Log the ismaster if available
+      if (self.s.logger.isInfo()) {
+        self.s.logger.info(
+          f('server %s connected with ismaster [%s]', self.name, JSON.stringify(self.ismaster))
+        );
+      }
 
-          // Ensure no error emitted after initial connect when reconnecting
-          self.initialConnect = false;
-          // Save the ismaster
-          self.ismaster = result.result;
-
-          // It's a proxy change the type so
-          // the wireprotocol will send $readPreference
-          if (self.ismaster.msg === 'isdbgrid') {
-            self._type = 'mongos';
-          }
-          // Add the correct wire protocol handler
-          self.wireProtocolHandler = configureWireProtocolHandler(self, self.ismaster);
-          // Have we defined self monitoring
-          if (self.s.monitoring) {
-            self.monitoringProcessId = setTimeout(
-              monitoringProcess(self),
-              self.s.monitoringInterval
-            );
-          }
-
-          // Emit server description changed if something listening
-          sdam.emitServerDescriptionChanged(self, {
-            address: self.name,
-            arbiters: [],
-            hosts: [],
-            passives: [],
-            type: sdam.getTopologyType(self)
-          });
-
-          if (!self.s.inTopology) {
-            // Emit topology description changed if something listening
-            sdam.emitTopologyDescriptionChanged(self, {
-              topologyType: 'Single',
-              servers: [
-                {
-                  address: self.name,
-                  arbiters: [],
-                  hosts: [],
-                  passives: [],
-                  type: sdam.getTopologyType(self)
-                }
-              ]
-            });
-          }
-
-          // Log the ismaster if available
-          if (self.s.logger.isInfo()) {
-            self.s.logger.info(
-              f('server %s connected with ismaster [%s]', self.name, JSON.stringify(self.ismaster))
-            );
-          }
-
-          // Emit connect
-          self.emit('connect', self);
-        }
-      );
+      // Emit connect
+      self.emit('connect', self);
     } else if (
       event === 'error' ||
       event === 'parseError' ||
@@ -554,7 +421,7 @@ var eventHandler = function(self, event) {
 
       // On first connect fail
       if (
-        self.s.pool.state === 'disconnected' &&
+        ['disconnected', 'connecting'].indexOf(self.s.pool.state) !== -1 &&
         self.initialConnect &&
         ['close', 'timeout', 'error', 'parseError'].indexOf(event) !== -1
       ) {
@@ -589,8 +456,6 @@ var eventHandler = function(self, event) {
 
 /**
  * Initiate server connect
- * @method
- * @param {array} [options.auth=null] Array of auth options to apply on connect
  */
 Server.prototype.connect = function(options) {
   var self = this;
@@ -630,12 +495,17 @@ Server.prototype.connect = function(options) {
     address: self.name
   });
 
-  // Connect with optional auth settings
-  if (options.auth) {
-    self.s.pool.connect.apply(self.s.pool, options.auth);
-  } else {
-    self.s.pool.connect();
-  }
+  self.s.pool.connect();
+};
+
+/**
+ * Authenticate the topology.
+ * @method
+ * @param {MongoCredentials} credentials The credentials for authentication we are using
+ * @param {authResultCallback} callback A callback function
+ */
+Server.prototype.auth = function(credentials, callback) {
+  if (typeof callback === 'function') callback(null, null);
 };
 
 /**
@@ -755,7 +625,7 @@ Server.prototype.command = function(ns, cmd, options, callback) {
     return callback(new MongoError(`server ${this.name} does not support collation`));
   }
 
-  self.wireProtocolHandler.command(self, ns, cmd, options, callback);
+  wireProtocol.command(self, ns, cmd, options, callback);
 };
 
 /**
@@ -786,7 +656,7 @@ Server.prototype.insert = function(ns, ops, options, callback) {
   ops = Array.isArray(ops) ? ops : [ops];
 
   // Execute write
-  return self.wireProtocolHandler.insert(self, ns, ops, options, callback);
+  return wireProtocol.insert(self, ns, ops, options, callback);
 };
 
 /**
@@ -821,7 +691,7 @@ Server.prototype.update = function(ns, ops, options, callback) {
   // Setup the docs as an array
   ops = Array.isArray(ops) ? ops : [ops];
   // Execute write
-  return self.wireProtocolHandler.update(self, ns, ops, options, callback);
+  return wireProtocol.update(self, ns, ops, options, callback);
 };
 
 /**
@@ -856,7 +726,7 @@ Server.prototype.remove = function(ns, ops, options, callback) {
   // Setup the docs as an array
   ops = Array.isArray(ops) ? ops : [ops];
   // Execute write
-  return self.wireProtocolHandler.remove(self, ns, ops, options, callback);
+  return wireProtocol.remove(self, ns, ops, options, callback);
 };
 
 /**
@@ -885,52 +755,6 @@ Server.prototype.cursor = function(ns, cmd, options) {
   return new FinalCursor(this.s.bson, ns, cmd, options, topology, this.s.options);
 };
 
-/**
- * Logout from a database
- * @method
- * @param {string} db The db we are logging out from
- * @param {authResultCallback} callback A callback function
- */
-Server.prototype.logout = function(dbName, callback) {
-  this.s.pool.logout(dbName, callback);
-};
-
-/**
- * Authenticate using a specified mechanism
- * @method
- * @param {string} mechanism The Auth mechanism we are invoking
- * @param {string} db The db we are invoking the mechanism against
- * @param {...object} param Parameters for the specific mechanism
- * @param {authResultCallback} callback A callback function
- */
-Server.prototype.auth = function(mechanism, db) {
-  var self = this;
-
-  if (mechanism === 'default') {
-    mechanism = getDefaultAuthMechanism(self.ismaster);
-  }
-
-  // Slice all the arguments off
-  var args = Array.prototype.slice.call(arguments, 0);
-  // Set the mechanism
-  args[0] = mechanism;
-  // Get the callback
-  var callback = args[args.length - 1];
-
-  // If we are not connected or have a disconnectHandler specified
-  if (disconnectHandler(self, 'auth', db, args, {}, callback)) {
-    return;
-  }
-
-  // Do not authenticate if we are an arbiter
-  if (this.lastIsMaster() && this.lastIsMaster().arbiterOnly) {
-    return callback(null, true);
-  }
-
-  // Apply the arguments to the pool
-  self.s.pool.auth.apply(self.s.pool, args);
-};
-
 /**
  * Compare two server instances
  * @method
@@ -954,6 +778,10 @@ Server.prototype.connections = function() {
 
 /**
  * Selects a server
+ * @method
+ * @param {function} selector Unused
+ * @param {ReadPreference} [options.readPreference] Unused
+ * @param {ClientSession} [options.session] Unused
  * @return {Server}
  */
 Server.prototype.selectServer = function(selector, options, callback) {
@@ -974,8 +802,11 @@ var listeners = ['close', 'error', 'timeout', 'parseError', 'connect'];
  * @param {boolean} [options.emitDestroy=false] Emit destroy event on destroy
  * @param {boolean} [options.force=false] Force destroy the pool
  */
-Server.prototype.destroy = function(options) {
-  if (this._destroyed) return;
+Server.prototype.destroy = function(options, callback) {
+  if (this._destroyed) {
+    if (typeof callback === 'function') callback(null, null);
+    return;
+  }
 
   options = options || {};
   var self = this;
@@ -991,6 +822,7 @@ Server.prototype.destroy = function(options) {
   // No pool, return
   if (!self.s.pool) {
     this._destroyed = true;
+    if (typeof callback === 'function') callback(null, null);
     return;
   }
 
@@ -1026,7 +858,7 @@ Server.prototype.destroy = function(options) {
   }
 
   // Destroy the pool
-  this.s.pool.destroy(options.force);
+  this.s.pool.destroy(options.force, callback);
   this._destroyed = true;
 };
 
